/* libhttpd.h - defines for libhttpd
**
** Copyright (C) 1995 by Jef Poskanzer <jef@acme.com>.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/

#ifndef _LIBHTTPD_H_
#define _LIBHTTPD_H_

#include <sys/types.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>


/* A few convenient defines. */

#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif
#define NEW(t,n) ((t*) malloc( sizeof(t) * (n) ))
#define RENEW(o,t,n) ((t*) realloc( (void*) o, sizeof(t) * (n) ))


/* The httpd structs. */

typedef struct {
    char* hostname;
    struct in_addr host_addr;
    int port;
    char* cgi_pattern;
    char* cwd;
    int fd;
    } httpd_server;

typedef struct {
    int initialized;
    httpd_server* hs;
    struct in_addr client_addr;
    int method;
    int status;
    off_t bytes;
    char* encodedurl;
    char* decodedurl;
    char* protocol;
    char* origfilename;
    char* expnfilename;
    char* encodings;
    char* pathinfo;
    char* query;
    char* altdir;
    char* referer;
    char* useragent;
    char* accept;
    char* accepte;
    char* cookie;
    char* contenttype;
    int maxencodedurl, maxdecodedurl, maxprotocol, maxorigfilename,
	maxexpnfilename, maxencodings, maxpathinfo, maxquery, maxaltdir,
	maxreferer, maxuseragent, maxaccept, maxaccepte, maxcookie,
	maxcontenttype;
    time_t if_modified_since;
    int contentlength;
    char* type;		/* not malloc()ed */
    int mime_flag;
    struct stat sb;
    int fd;
    FILE* rfp;
    int file_fd;
    } httpd_conn;
#define METHOD_GET 1
#define METHOD_HEAD 2
#define METHOD_POST 3


/* Initializes.  Does the socket(), bind(), and listen().   Returns an
** httpd_server* which includes a socket fd that you can select() on.
** Return (httpd_server*) 0 on error.
*/
extern httpd_server* httpd_initialize( char* hostname, int port, char* cgi_pattern, char* cwd );

/* Call to shut down. */
extern void httpd_terminate( httpd_server* hs );


/* When the socket fd is ready to read, call this.  It does the accept() and
** returns an httpd_conn* which includes the URL and a stream to write
** the response to.  Returns -1 if there was no valid request, else 0.
**
** In order to minimize malloc()s, the caller passes in the httpd_conn.
** The caller is also responsible for setting initialized to zero before the
** first call using each different httpd_conn.
*/
extern int httpd_get_conn( httpd_server* hs, httpd_conn* hc );

/* Starts sending data back to the client.  In some cases (directories,
** CGI programs), finishes sending by itself - in those cases, hc->file_fd
** is <0.  If there is more data to be sent, then hc->file_rfd is a file
** descriptor for the file to send.
**
** Returns -1 on error.
*/
extern int httpd_start_request( httpd_conn* hc );

/* Call this to close down a connection and free the data.  A fine point,
if you fork() with a connection open you should still call this in the
** parent process - the connection will stay open in the child.
*/
extern void httpd_close_conn( httpd_conn* hc );


/* Send an error message back to the client. */
extern void httpd_send_err( httpd_conn* hc, int status, char* title, char* form, char* arg );

/* Some error messages. */
extern char* httpd_err503title;
extern char* httpd_err503form;

/* Generates a string representation of a method number. */
extern char* httpd_method_str( int method );

/* Sets the allowed number of file descriptors to the maximum and returns it. */
extern int httpd_get_nfiles( void );

#endif /* _LIBHTTPD_H_ */
